"use strict";
/**
 *  Bitloops Language CLI
 *  Copyright (C) 2022 Bitloops S.A.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 *  For further information you can contact legal(at)bitloops.com.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const ora_1 = require("ora");
const copyright_js_1 = require("./copyright.js");
const index_js_1 = require("../functions/index.js");
const fileOperations_js_1 = require("../helpers/fileOperations.js");
const oraUtils_js_1 = require("../utils/oraUtils.js");
const inquirer_js_1 = require("../utils/inquirer.js");
const transpile_js_1 = require("../functions/transpile.js");
const generateTargetFiles_js_1 = require("../functions/generateTargetFiles.js");
const SETUP_FILE_EXTENSION = 'setup.bl';
// 😑 question messages don't work with chalk
const questions = [
    {
        // type: 'input',
        name: 'sourceDirPath',
        message: '📦 Where are your source files located?',
        default: '.',
    },
    {
        // type: 'input',
        name: 'targetDirPath',
        message: '🎯 Where would you like your generated files to be exported?',
        default: './output',
    },
];
const transpile = async (source) => {
    console.log();
    console.log(copyright_js_1.copyrightSnippet);
    console.log();
    const answers = [];
    for (const q of questions) {
        answers.push(await (0, inquirer_js_1.inquirerFuzzy)(q, source));
    }
    const [sourceDirPath, targetDirPath] = answers;
    if (!fs_1.default.existsSync(targetDirPath)) {
        fs_1.default.mkdirSync(targetDirPath);
    }
    else {
        const overwriteAccepted = await (0, inquirer_js_1.inquirerSimpleConfirm)('Overwrite', '⚠️  Target directory already exists. Overwrite?', 'n');
        if (!overwriteAccepted) {
            return;
        }
        await (0, index_js_1.deleteFolderContent)(targetDirPath);
    }
    let throbber;
    try {
        const boundedContextModules = (0, index_js_1.getBoundedContextModules)(sourceDirPath);
        const dirEntries = fs_1.default.readdirSync(sourceDirPath, { withFileTypes: true });
        const filesNames = dirEntries
            .filter((entry) => entry.isFile())
            .map((fileName) => fileName.name);
        const atLeastOneSetupFileExists = filesNames.some((file) => file.toLowerCase().endsWith(SETUP_FILE_EXTENSION));
        const atLeastOneModuleExists = Object.values(boundedContextModules).some((modules) => modules.length > 0);
        // TODO Gather all errors and display them at once
        if (!atLeastOneSetupFileExists) {
            (0, inquirer_js_1.printError)('No setup file found. Please create a setup file in the root of your project.');
            process.exit(1);
        }
        if (!atLeastOneModuleExists) {
            (0, inquirer_js_1.printError)('No modules found. Please create a module inside one of your bounded contexts.');
            process.exit(1);
        }
        // TODO Check if the output directory exists and if it does ask if the user wants to overwrite it
        (0, fileOperations_js_1.clearFolder)(targetDirPath);
        throbber = (0, ora_1.default)((0, oraUtils_js_1.purpleColor)('🔨 Transpiling... ')).start();
        const transpiledCode = await (0, transpile_js_1.transpileCode)(boundedContextModules, sourceDirPath);
        (0, oraUtils_js_1.stopSpinner)(throbber, (0, oraUtils_js_1.greenColor)('Transpiled'), '🔨');
        throbber = (0, ora_1.default)((0, oraUtils_js_1.purpleColor)('🕒 Writing system files to disk...')).start();
        (0, generateTargetFiles_js_1.writeTranspiledCode)(transpiledCode, targetDirPath);
        (0, oraUtils_js_1.stopSpinner)(throbber, (0, oraUtils_js_1.greenColor)('System files written'), '⏰');
        // console.log(greenColor('Project generated successfully!'));
        console.log((0, oraUtils_js_1.greenColor)(oraUtils_js_1.TAB + '🦎 Project generated successfully!\n'));
    }
    catch (err) {
        console.log(err);
        console.error((0, oraUtils_js_1.redColor)(oraUtils_js_1.TAB + '❌ ' + err));
        throbber.stop();
    }
};
exports.default = transpile;
//# sourceMappingURL=transpile.js.map